#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <string.h>

#include "mn2fw.h"
#include "mn2lcd.h"
#include "mn2kbd.h"
#include "mn2host.h"

/*
 * Host interface
 */
#define HREQ_PORT   PORTC
#define HREQ_DDR    DDRC
#define HREQ_BIT    0    // IRQ to host, not used

#define MISO_DDR    DDRB
#define MISO_BIT    4

static unsigned char spi_cnt;
static unsigned char spi_cmd;
static unsigned char spi_rsp;

unsigned char volatile dsp_hupd;
unsigned char volatile dsp_hclr;
unsigned char volatile dsp_hmod;
unsigned char volatile dsp_hspd;

static unsigned char dsp_hpos;
static unsigned char dsp_hatt;

/*!
 * \brief SPI interrupt handler.
 */
ISR(SPI_STC_vect)
{
    unsigned char cmd = SPDR;

    SPDR = spi_rsp;

    /*
     * State 0: Expecting a new command.
     */
    if (spi_cnt == 0) {
        spi_cmd = cmd;
        if (cmd >= MEDIA2_CMD_FIRST && cmd <= MEDIA2_CMD_LAST) {
            spi_rsp = 0x00;
        } else {
            spi_rsp = 0xFF;
        }
        spi_cnt++;
    }

    /*
     * State 1: Expecting command parameter.
     */
    else if (spi_cnt == 1) {
        if (spi_cmd == MEDIA2_CMD_RESET) {
            /* TODO */
        }
        else if (spi_cmd == MEDIA2_CMD_MAJOR_VERSION) {
            spi_rsp = VERSION_MAJOR;
        }
        else if (spi_cmd == MEDIA2_CMD_MINOR_VERSION) {
            spi_rsp = VERSION_MINOR;
        }
        else if (spi_cmd == MEDIA2_CMD_RELEASE) {
            spi_rsp = VERSION_RELEASE;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_MODE) {
            spi_rsp = dsp_lmod;
            dsp_hmod = cmd;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_STATUS) {
            spi_rsp = dsp_lmod;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_CLEAR) {
            spi_rsp = 0;
            dsp_hatt = 0;
            dsp_hclr = 1;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_POS) {
            spi_rsp = dsp_hpos;
            dsp_hpos = cmd;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_WRITE) {
            spi_rsp = dsp_vtxt[dsp_hpos];
            if (spi_rsp != cmd || dsp_hatt != dsp_vatt[dsp_hpos]) {
                dsp_vtxt[dsp_hpos] = cmd;
                dsp_vatt[dsp_hpos] = dsp_hatt;
                dsp_hupd = 1;
            }
            dsp_hpos++;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_READ) {
            spi_rsp = dsp_vtxt[dsp_hpos];
            dsp_hpos++;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_ATTRIB) {
            spi_rsp = dsp_hatt;
            dsp_hatt = cmd;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_SCROLL) {
            /* 0 is slow, 255 is fast. */
            spi_rsp = dsp_hspd;
            dsp_hspd = cmd;
        }
        else if (spi_cmd == MEDIA2_CMD_DISPLAY_COMMAND) {
        }
        else if (spi_cmd == MEDIA2_CMD_KBD_CODE) {
            spi_rsp = kbd_code;
            kbd_code = 0;
        }
        spi_cnt++;
    }

    /*
     * State 2: Dummy.
     */
    else if (spi_cnt == 2) {
        spi_rsp = 0xFF;
        spi_cnt++;
    }

    /*
     * State 3: Master fetched response.
     */
    else {
        spi_rsp = 0xFF;
        spi_cnt = 0;
    }
#if 0
    SPDR = 0x33;
    if ((unsigned char) (wc + 1) != rc) {
        spibuff[wc] = SPDR;
        SPDR = 0x22;
        wc++;
    }
#endif
}

void Mn2HostInit(void)
{
    /* Set MISO output, all others input */
    MISO_DDR = 1 << MISO_BIT;
    /* Enable SPI */
    SPCR |= 1 << SPE;
    /* Enable SPI Interrupt */
    SPCR |= 1 << SPIE;
}
