/*!
 * Copyright (C) 2007 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log$
 */

#include <cfg/os.h>
#include <dev/board.h>
#include <sys/timer.h>

#include <dev/vs10xx.h>

#include <mn2ctrl.h>

#include "media2spi.h"

#if defined(__AVR__)
#define MEDIANUT2_MCU_SPI0_DEVICE   2
#else
#endif

#ifndef MEDIANUT2_MCU_SPI_MODE
#define MEDIANUT2_MCU_SPI_MODE   0
#endif

#ifndef MEDIANUT2_MCU_SPI_RATE
#define MEDIANUT2_MCU_SPI_RATE   800000
#endif

#if defined(MEDIANUT2_MCU_SPI0_DEVICE) /* SPI device. */

#include <dev/sppif0.h>
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define TargetReset(act)    Sppi0ChipReset(MEDIANUT2_MCU_SPI0_DEVICE, act)
#else
#define TargetReset(act)    Sppi0ChipReset(MEDIANUT2_MCU_SPI0_DEVICE, !act)
#endif
#define SpiSetMode()        Sppi0SetMode(MEDIANUT2_MCU_SPI0_DEVICE, MEDIANUT2_MCU_SPI_MODE)
#define SpiSetSpeed()       Sppi0SetSpeed(MEDIANUT2_MCU_SPI0_DEVICE, MEDIANUT2_MCU_SPI_RATE)
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define SpiSelect()         Sppi0SelectDevice(MEDIANUT2_MCU_SPI0_DEVICE)
#define SpiDeselect()       Sppi0DeselectDevice(MEDIANUT2_MCU_SPI0_DEVICE)
#else
#define SpiSelect()         Sppi0NegSelectDevice(MEDIANUT2_MCU_SPI0_DEVICE)
#define SpiDeselect()       Sppi0NegDeselectDevice(MEDIANUT2_MCU_SPI0_DEVICE)
#endif
#define SpiByte             Sppi0Byte

#elif defined(MEDIANUT2_MCU_SBBI0_DEVICE) /* SPI device. */

#include <dev/sbbif0.h>
#define MEDIANUT2_MCU_CS        MEDIANUT2_MCU_SBBI0_DEVICE
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define TargetReset(act)    Sbbi0ChipReset(MEDIANUT2_MCU_CS, act)
#else
#define TargetReset(act)    Sbbi0ChipReset(MEDIANUT2_MCU_CS, !act)
#endif
#define SpiSetMode()        Sbbi0SetMode(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_MODE)
#define SpiSetSpeed()       Sbbi0SetSpeed(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_RATE)
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define SpiSelect()         Sbbi0SelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi0DeselectDevice(MEDIANUT2_MCU_CS)
#else
#define SpiSelect()         Sbbi0NegSelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi0NegDeselectDevice(MEDIANUT2_MCU_CS)
#endif
#define SpiByte             Sbbi0Byte

#elif defined(MEDIANUT2_MCU_SBBI1_DEVICE) /* SPI device. */

#include <dev/sbbif1.h>
#define MEDIANUT2_MCU_CS        MEDIANUT2_MCU_SBBI0_DEVICE
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define TargetReset(act)    Sbbi1ChipReset(MEDIANUT2_MCU_CS, act)
#else
#define TargetReset(act)    Sbbi1ChipReset(MEDIANUT2_MCU_CS, !act)
#endif
#define SpiSetMode()        Sbbi1SetMode(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_MODE)
#define SpiSetSpeed()       Sbbi1SetSpeed(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_RATE)
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define SpiSelect()         Sbbi1SelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi1DeselectDevice(MEDIANUT2_MCU_CS)
#else
#define SpiSelect()         Sbbi1NegSelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi1NegDeselectDevice(MEDIANUT2_MCU_CS)
#endif
#define SpiByte             Sbbi1Byte

#elif defined(MEDIANUT2_MCU_SBBI2_DEVICE) /* SPI device. */

#include <dev/sbbif2.h>
#define MEDIANUT2_MCU_CS        MEDIANUT2_MCU_SBBI0_DEVICE
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define TargetReset(act)    Sbbi2ChipReset(MEDIANUT2_MCU_CS, act)
#else
#define TargetReset(act)    Sbbi2ChipReset(MEDIANUT2_MCU_CS, !act)
#endif
#define SpiSetMode()        Sbbi2SetMode(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_MODE)
#define SpiSetSpeed()       Sbbi2SetSpeed(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_RATE)
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define SpiSelect()         Sbbi2SelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi2DeselectDevice(MEDIANUT2_MCU_CS)
#else
#define SpiSelect()         Sbbi2NegSelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi2NegDeselectDevice(MEDIANUT2_MCU_CS)
#endif
#define SpiByte             Sbbi2Byte

#elif defined(MEDIANUT2_MCU_SBBI3_DEVICE) /* SPI device. */

#include <dev/sbbif3.h>
#define MEDIANUT2_MCU_CS        MEDIANUT2_MCU_SBBI0_DEVICE
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define TargetReset(act)    Sbbi3ChipReset(MEDIANUT2_MCU_CS, act)
#else
#define TargetReset(act)    Sbbi3ChipReset(MEDIANUT2_MCU_CS, !act)
#endif
#define SpiSetMode()        Sbbi3SetMode(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_MODE)
#define SpiSetSpeed()       Sbbi3SetSpeed(MEDIANUT2_MCU_CS, MEDIANUT2_MCU_SPI_RATE)
#if defined(MEDIANUT2_MCU_SELECT_ACTIVE_HIGH)
#define SpiSelect()         Sbbi3SelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi3DeselectDevice(MEDIANUT2_MCU_CS)
#else
#define SpiSelect()         Sbbi3NegSelectDevice(MEDIANUT2_MCU_CS)
#define SpiDeselect()       Sbbi3NegDeselectDevice(MEDIANUT2_MCU_CS)
#endif
#define SpiByte             Sbbi3Byte

#endif /* SPI device. */


static u_int display_vcols;

/*!
 * \brief Send programming command and return response.
 *
 * \param cmd 4-byte command.
 *
 * \return 4-byte response.
 */
u_char Media2Cmd(u_char cmd, u_char parm)
{
    u_char rc;
    u_char ief;
    u_char tef;

    ief = VsPlayerInterrupts(0);
    tef = VsPlayerThrottle(1);

    SpiSelect();

    SpiByte(cmd);
    SpiByte(parm);
    NutDelay(1);
    SpiByte(0xFF);
    NutDelay(1);
    rc = SpiByte(0xFF);

    SpiDeselect();

    VsPlayerThrottle(tef);
    VsPlayerInterrupts(ief);

    return rc;
}

int Media2DisplayMode(ureg_t mode)
{
    if (SpiSetMode()) {
        return -1;
    }
    SpiSetSpeed();
    SpiDeselect();

    Media2Cmd(MEDIA2_CMD_DISPLAY_MODE, (u_char)mode);
    if (Media2Cmd(MEDIA2_CMD_DISPLAY_STATUS, 0) != (u_char)mode) {
        return -1;
    }

    /*
     * Medianut 2 provides a 256 byte virtual display buffer. Calculate
     * the number of virtual columns per line based on the number of
     * rows.
     */
    if (mode == LCD_MODE_4X16 || mode == LCD_MODE_4X20) {
        /* 4 rows. */
        display_vcols = 64;
    }
    else if (mode == LCD_MODE_1X8 || mode == LCD_MODE_1X16 || mode == LCD_MODE_1X20) {
        /* 1 row. */
        display_vcols = 256;
    }
    else {
        /* 2 rows by default. */
        display_vcols = 128;
    }
    return 0;
}

void Media2DisplayScrollMode(ureg_t mode)
{
    Media2Cmd(MEDIA2_CMD_DISPLAY_SCROLL, (u_char)mode);
}

void Media2DisplayClear(void)
{
    Media2Cmd(MEDIA2_CMD_DISPLAY_CLEAR, 0);
}

void Media2DisplayPos(ureg_t row, ureg_t col)
{
    Media2Cmd(MEDIA2_CMD_DISPLAY_POS, (u_char)(row * display_vcols + col));
}

void Media2DisplayWrite(CONST char *data)
{
	while (*data) {
        Media2Cmd(MEDIA2_CMD_DISPLAY_WRITE, (u_char)*data);
        data++;
	}
}

void Media2DisplayWriteRow(ureg_t row, CONST char *str)
{
    Media2DisplayWritePos(row, 0, str);
}

void Media2DisplayWritePos(ureg_t row, ureg_t col, CONST char *str)
{
    Media2DisplayPos(row, col);
    Media2DisplayWrite(str);
}

void Media2DisplayAttrib(ureg_t attrib)
{
    Media2Cmd(MEDIA2_CMD_DISPLAY_ATTRIB, (u_char)attrib);
}

